#include <stdio.h>

  /* Parcijalne sume harmonijskog reda zbrajanjem unaprijed i unatrag,
     u double i float tipovima. */

  /* Napomena: program je NAMJERNO napisan tako da se SVI medjurezultati
     (poput 1.0 / i) spremaju u varijable odgovarajuceg tipa, zato da se
     POKUSA dobiti konzistentno zaokruzivanje u pripadnom tipu, prilikom
     cijelog racuna.
       Nazalost, BEZ obzira na to, rezultati BITNO ovise o OPCIJAMA
     koje su zadane prilikom kompajliranja (naravno, i o kompajleru).
       Dobiveni rezultati mogu biti TOCNIJI nego sto bi trebali biti,
     jer se stvarno racunanje i zbrajanje clanova moze raditi u registrima
     bitno VECE tocnosti nego sto je zadani tip, a tek FINALNI rezultat
     se zaokruzuje na zadani tip (double, float). */

int main(void)
{
    int n = 1000000;    /* Broj clanova reda u sumi. */
    int i;
    float suma_f, clan_f;
    double suma_d, clan_d;
    double err, rel_err;

        /* Egzaktna suma, dobivena posebnim algoritmom zbrajanja
           u jos visoj tocnosti (extended). */

    double exact_sum = 14.3927267228657236;


    printf(" Maksimalno tocna suma (extended):\n");
    printf("        S = %18.15f\n", exact_sum);


        /* Double, zbrajanje unaprijed, pa unatrag. */

    printf("\n Double:\n");

    suma_d = 0.0;
    for (i = 1; i <= n; ++i) {
        clan_d = 1.0 / ( (double) i );
        suma_d = suma_d + clan_d;
    }
    err = exact_sum - suma_d;
    rel_err = err / exact_sum;
    printf("   S (->) = %18.15f   abs = %13.5e  rel = %13.5e\n",
           suma_d, err, rel_err);

    suma_d = 0.0;
    for (i = n; i >= 1; --i) {
        clan_d = 1.0 / ( (double) i );
        suma_d = clan_d + suma_d;
    }
    err = exact_sum - suma_d;
    rel_err = err / exact_sum;
    printf("   S (<-) = %18.15f   abs = %13.5e  rel = %13.5e\n",
           suma_d, err, rel_err);


        /* Float, zbrajanje unaprijed, pa unatrag. */

    printf("\n Float:\n");

    suma_f = 0.0f;
    for (i = 1; i <= n; ++i) {
        clan_f = 1.0f / ( (float) i );
        suma_f = suma_f + clan_f;
    }
    err = exact_sum - suma_f;
    rel_err = err / exact_sum;
    printf("   S (->) = %18.15f   abs = %13.5e  rel = %13.5e\n",
           suma_f, err, rel_err);

    suma_f = 0.0f;
    for (i = n; i >= 1; --i) {
        clan_f = 1.0f / ( (float) i );
        suma_f = clan_f + suma_f;
    }
    err = exact_sum - suma_f;
    rel_err = err / exact_sum;
    printf("   S (<-) = %18.15f   abs = %13.5e  rel = %13.5e\n",
           suma_f, err, rel_err);

    return 0;
}
