#include <stdio.h>
#include <stdlib.h>

  /* Binarno potenciranje:
       Wirthova varijanta (A+DS=P). Inicijalizacija potencije na 1,
       ali se kvadriranje radi u blokovima nul-znamenki u n.
  */

/****************************************************************************/

    /* Binarno potenciranje x^n. */

    /* Pretpostavka (bez provjere): n >= 0 i x != 0. */

    /* Rezultat se inicijalizira na 1, sto je korektno za n = 0.
       Za n > 0, prvo mnozenje je 1 * b, sto se moze izbjeci.
       Pogledati: N. Wirth, "Algorthms + Data Structures = Programs".
       Kvadriranje se radi u blokovima, bez dodatnog kvadriranja na kraju.
       Slozenost: O(lg n) mnozenja.
    */

double pow_bin(double x, int n)
{
    double pot = 1.0, b = x;

        /* Potenciranje kvadriranjem i mnozenjem. */

    while (n > 0) {

            /* Sve dok je (trenutni) n paran (zadnja binarna znamenka
               je jednaka 0), kvadriraj b i obrisi zadnju znamenku u n. */

            /* Za provjeru parnosti od n, umjesto n % 2 == 0,
               koristimo test zadnjeg bita u n, tj. (n & 1) == 0. */

        while ((n & 1) == 0) {

                /* Kvadriraj b. */

            b *= b;

                /* Obrisi zadnju binarnu znamenku u n, tj. podijeli n s 2.
                   Brza realizacija je pomak za jedno mjesto udesno (>> 1). */

            n >>= 1;    /* n /= 2; */
        }


            /* Zadnja binarna znamenka (trenutnog) n je jednaka 1, tj.
               n je neparan. Akumuliraj produkt u pot (pomnozi pot s b). */

        pot *= b;

            /* Promijeni zadnju binarnu znamenku (trenutnog) n s 1 na 0
               (nemoj ju brisati), tj. smanji n za 1. */

        --n;
    }

    return pot;
}

    /* Cjelobrojno potenciranje x^n.
       Koristi funkciju za x^abs(n). */

double int_pow(double x, int n)
{
    double pot;
    int neg;

        /* Provjera x = 0. */
    if (x == 0.0) return 0.0;

        /* Zapamti predznak od n. */
    neg = n < 0;
    n = abs(n);

        /* Potenciranje kvadriranjem i mnozenjem
           (binarno potenciranje). */
    pot = pow_bin(x, n);

    if (neg) pot = 1.0 / pot;
    return pot;
}

int main(void)
{
    double x;
    int n;

    x = 2.0;
    for (n = 0; n <= 16; ++n)
        printf(" Potencija %3.1f na %2d = %7.1f\n",
                 x, n, int_pow(x, n) );

    n = 1000000000;     /* n = 10^9 */
    x = 1.000000001;    /* x = 1 + 1.0/n */

    printf(" Potencija %11.9f na %d = %11.9f\n",
           x, n, int_pow(x, n) );

    return 0;
}
