#include <stdio.h>

/****************************************************************************/

  /* Insertion sort algoritam (sortiranje umetanjem).
     Sortirani dio je spijeda. Binarno trazenje, onda pomaci.
     Izbjegava nepotrebna kopiranja ako je novi element najveci.
  */

/****************************************************************************/

  /* Uzlazno sortiranje (sortiranim) umetanjem.
     Sortirani dio je sprijeda.
     Umjesto sekvencijalnog, koristi binarno trazenje.
     Ako je sljedeci element veci, ne radi nista (nema kopiranja). */

void insertion_sort(int x[], int n)
{
    int i, j, l, d, m, temp;

    for (i = 1; i < n; ++i) {

            /* Ako je x[i] >= x[i - 1], onda nemamo sto za raditi,
               jer je i polje x[0], ..., x[i] uzlazno sortirano,
               tj. x[i] je vec na korektnom mjestu. */

        if (x[i] < x[i - 1]) {
            temp = x[i];

                /* Usporedba x[i] s najmanjim x[0]. */

            if (temp >= x[0]) {

                    /* Binarno trazenje u x[0], ..., x[i - 1].
                       Trazimo indeks d takav da je x[d - 1] <= temp < x[d].
                       Uz inicijalizaciju l = 0, d = i - 1, uvijek vrijedi
                       x[l] <= temp < x[d]. Stajemo kad je l + 1 = d. */

                l = 0;
                d = i - 1;
                while (l + 1 < d) {
                    m = (l + d) / 2;
                    if (temp < x[m])
                        d = m;
                    else    /* x[m] <= temp. */
                        l = m;
                }
            }
            else    /* x[i] < x[0], tj. x[i] je novi najmanji. */
                d = 0;

                /* Pomak desnog dijela sortiranog polja za jedno mjesto udesno
                   x[d], ..., x[i - 1] --> x[d + 1], ..., x[i]. */
            for (j = i; j > d; --j)
                x[j] = x[j - 1];

            x[j] = temp;    /* Moze i x[d] = temp. */

        }    /* Kraj if (x[i] < x[i - 1]). */

    }    /* Kraj for po i. */

    return;
}

/****************************************************************************/

int main(void)
{
    int i, n;
    int x[] = {42, 12, 55, 94, 18, 44, 67};

    n = 7;
    insertion_sort(x, n);

    printf("\n Sortirano polje x:\n");
    for (i = 0; i < n; ++i) {
        printf(" x[%d] = %d\n", i, x[i]);
    }

    return 0;
}
